@@ -11,21 +11,22 @@ from subprocess import check_output, run
 import hvac
 import yaml
 
+from .exception import ChamberError
 from .vault import VaultSecrets
 
-
-class ChamberError(Exception):
-    pass
-
-
 EXEC_WAIT = True
 
 
 class Chamber:
-    def __init__(self, config, debug, echo):
+    def __init__(self, *, config, debug, echo, require_exists):
+        self.config = config
         secrets_dir = config["dir"]
         self.secrets_dir = Path(secrets_dir)
         self.echo = echo
+        self.require_service = require_exists
+        self.require_key = require_exists
+        self.force_lower_services = False
+        self.force_lower_keys = False
 
     def __enter__(self):
         return self
@@ -33,6 +34,12 @@ class Chamber:
     def __exit__(self, _, ex, tb):
         pass
 
+    def _secret_not_found(self, service, key):
+        return f"Error: secret not found: '{service}/{key}'"
+
+    def _service_not_found(self, service):
+        return f"Error: service not found: '{service}'"
+
     def _echo(self, msg):
         return self.echo(msg)
 
@@ -48,27 +55,77 @@ class Chamber:
 
     def delete(self, service, key):
         """Delete a secret, including all versions"""
-        self._delete(service, key)
+        service, key = self._verify_key(service, key)
+        if service and key:
+            self._delete(service, key)
         return 0
 
+    def _is_service(self, service):
+        """return True if service exists, else False"""
+        return service in self._list_services()
+
+    def _is_secret(self, service, key):
+        """return True if service exists and contains key, else False"""
+        return self._is_service(service) and (key in self._secrets(service))
+
+    def _verify_service(self, service, condition=None):
+        """raise an error if require_service and condition is false"""
+        if self.force_lower_services:
+            service = service.lower()
+        if condition is None:
+            condition = self._is_service(service)
+        else:
+            condition = bool(condition)
+        if bool(service) and condition:
+            pass
+        elif self.require_service:
+            raise ChamberError(self._service_not_found(service))
+        else:
+            service = None
+        return service
+
+    def _verify_key(self, service, key, condition=None):
+        """raise an error if require_service and condition is false"""
+        service = self._verify_service(service)
+        if service is None:
+            key = None
+        else:
+            if self.force_lower_keys:
+                key = key.lower()
+            if condition is None:
+                condition = self._is_secret(service, key)
+            else:
+                condition = bool(condition)
+            if bool(key) and condition:
+                pass
+            elif self.require_key:
+                raise ChamberError(self._secret_not_found(service, key))
+            else:
+                key = None
+        return service, key
+
     def prune(self, service):
         """Prune a service, including all subkeys"""
-        path = service.split("/")
-        plen = len(path)
-        for _service in self._list_services():
-            _path = _service.split("/")
-            if path == _path[:plen]:
-                for _key in self._secrets(_service).keys():
-                    self.delete(_service, _key)
+        service = self._verify_service(service)
+        if service:
+            path = service.split("/")
+            plen = len(path)
+            for _service in self._list_services():
+                _path = _service.split("/")
+                if path == _path[:plen]:
+                    for _key in self._secrets(_service).keys():
+                        self.delete(_service, _key)
         return 0
 
     def env(self, service):
         """Print the secrets from the secrets directory in a format to export as environment variables"""  # noqa
-        secrets = self._secrets(service.lower())
-        self.echo("\n".join(sorted([self._export(k, v) for k, v in secrets.items()])))
+        service = self._verify_service(service)
+        if service:
+            secrets = self._secrets(service)
+            self.echo("\n".join(sorted([self._export(k, v) for k, v in secrets.items()])))
         return 0
 
-    def _exec(self, *, pristine, strict_value, services, cmd):
+    def _exec(self, *, services, cmd, pristine=False, strict_value='chamberme', child=True, buffer_output=True):
         """Executes a command with secrets loaded into the environment"""
         if not cmd:
             raise ChamberError("Error: must specify command to run. See usage: requires at least 1 arg(s), only received 0")  # noqa
@@ -84,24 +141,37 @@ class Chamber:
             env = {}
 
         for service in services:
-            secrets = self._secrets(service.lower())
-            for k, v in secrets.items():
-                env[k.upper()] = str(v)
+            service = self._verify_service(service)
+            if service:
+                secrets = self._secrets(service)
+                for k, v in secrets.items():
+                    env[k.upper()] = str(v)
 
         # if we have any strict_vars; raise exception if they have not been overwritten  # noqa
         for svar in strict_vars:
             if (svar not in env) or (env[svar] == strict_value):
-                raise ChamberError(f"parent env was expecting {svar}={strict_value}, but was not in store")  # noqa
+                raise ChamberError(f"Error: parent env was expecting {svar}={strict_value}, but was not in store")  # noqa
 
-        if EXEC_WAIT:
-            proc = run(cmd, env=env)
-            return proc.returncode
+        self.proc = self._exec_subprocess(child, buffer_output, env, cmd)
+        return self.proc.returncode
+
+    def _exec_subprocess(self, child, buffer_output, env, cmd):
+        if child:
+            if buffer_output:
+                proc = run(cmd, env=env, capture_output=True, text=True)
+            else:
+                proc = run(cmd, env=env)
+            return proc
         else:
             execvpe(cmd[0], cmd, env)
 
     def export(self, *, output_file, fmt, service, compact_json=False, sort_keys=True):
         """Exports parameters in the specified format"""
-        secrets = self._secrets(service.lower())
+        service = self._verify_service(service)
+        if service:
+            secrets = self._secrets(service)
+        else:
+            secrets = {}
         if fmt == "json":
             if compact_json:
                 out = json.dumps(secrets, separators=[",", ":"], sort_keys=sort_keys) + "\n"
@@ -158,9 +228,11 @@ class Chamber:
 
     def list(self, service):
         """List the secrets set for a service"""
-        service = service.lower()
-        secrets = self._list(service)
-
+        service = self._verify_service(service)
+        if service:
+            secrets = self._list(service)
+        else:
+            secrets = {}
         namelen = max(len(name) for name in secrets.keys())
         tabs = int(namelen / 8) + 1
         ltab = "\t" * tabs
@@ -175,33 +247,40 @@ class Chamber:
         """List services"""
         self.echo("Service")
         output = []
+        if self.force_lower_services and service_filter is not None:
+            service_filter = service_filter.lower()
         services = self._list_services()
         for service in services:
-            if service_filter is None or service.startswith(str(service_filter).lower()):
+            if service_filter is None or service.startswith(service_filter):
                 if include_secrets:
                     for secret in sorted(self._secrets(service)):
                         output.append(f"{service}/{secret}")
                 else:
                     output.append(service)
-
         for line in sorted(output):
             self.echo(line)
         return 0
 
     def read(self, service, key, quiet=False):
         """Read a specific secret from the parameter store"""
-        value, mtime, owner = self._read(service, key)
-        out = f"{key}\t{value}\t1\t{mtime}\t{owner}"
-        if quiet:
-            self.echo(value)
-        else:
-            self.echo("Key\tValue\tVersion\tLastModified\tUser")
-            self.echo(out)
+        service, key = self._verify_key(service, key)
+        if key:
+            value, mtime, owner = self._read(service, key)
+            out = f"{key}\t{value}\t1\t{mtime}\t{owner}"
+            if quiet:
+                self.echo(value)
+            else:
+                self.echo("Key\tValue\tVersion\tLastModified\tUser")
+                self.echo(out)
 
         return 0
 
     def write(self, service, key, value):
         """write a secret"""
+        if self.force_lower_services:
+            service = service.lower()
+        if self.force_lower_keys:
+            key = key.lower()
         if value == "-":
             value = sys.stdin.read()
         self._write(service, key, value)
@@ -209,8 +288,8 @@ class Chamber:
 
 
 class VaultChamber(Chamber):
-    def __init__(self, config, debug, echo):
-        super().__init__(config, debug, echo)
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
 
     def __enter__(self):
         self.secrets = VaultSecrets()
@@ -243,7 +322,10 @@ class VaultChamber(Chamber):
         try:
             metadata = self.secrets.get_metadata(service, key)
         except hvac.exceptions.InvalidPath as ex:
-            raise ChamberError("Error: secret not found") from ex
+            if self.require_key:
+                raise ChamberError(self._secret_not_found(service, key)) from ex
+            else:
+                return (None, None, None)
         timestamp = metadata["created_time"].split(".")[0]
         mtime = datetime.fromisoformat(timestamp)
         owner = "undefined"
@@ -253,7 +335,8 @@ class VaultChamber(Chamber):
     def _read(self, service, key):
         """return a secret (value, mtime, owner)"""
         _, mtime, owner = self._metadata(service, key)
-        return (self.secrets.get(service, key), mtime, owner)
+        value = self.secrets.get(service, key)
+        return (value, mtime, owner)
 
     def _delete(self, service, key):
         """delete a secret"""
@@ -262,15 +345,20 @@ class VaultChamber(Chamber):
 
 
 class EnvdirChamber(Chamber):
-    def __init__(self, config, debug, echo):
-        super().__init__(config, debug, echo)
-        self.secrets_dir = config["dir"]
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self.secrets_dir = self.config["dir"]
 
     def _secrets_dir(self, service):
         return self.secrets_dir / service
 
     def _service_name(self, service):
-        return str(service.relative_to(self.secrets_dir)).lower()
+        if self.force_lower_services:
+            service = service.lower()
+        ret = str(service.relative_to(self.secrets_dir))
+        if self.force_lower_services:
+            ret = ret.lower()
+        return ret
 
     def _listdirs(self, dirs, curdir):
         if dirs is None:
@@ -289,7 +377,7 @@ class EnvdirChamber(Chamber):
         owner = check_output(f"id -un {stat.st_uid}", shell=True).decode().strip()
         return mtime, owner
 
-    def _secrets(self, service):
+    def _secrets(self, service, require_exists=True):
         """return dict of secrets in a service"""
         secrets = self._secrets_dir(service)
         if secrets.is_dir():
@@ -312,7 +400,11 @@ class EnvdirChamber(Chamber):
 
     def _write(self, service, key, value):
         """write a secret"""
-        secret = self.secrets_dir / service.lower() / key.lower()
+        if self.force_lower_services:
+            service = service.lower()
+        if self.force_lower_keys:
+            key = key.lower()
+        secret = self.secrets_dir / service / key
         secret.parent.mkdir(parents=True, exist_ok=True)
         secret.write_text(value)
 
@@ -323,21 +415,25 @@ class EnvdirChamber(Chamber):
             value = secret.read_text()
             mtime, owner = self._stats(secret)
         except FileNotFoundError as ex:
-            raise ChamberError("Error: Failed to read: secret not found") from ex
+            raise ChamberError(self._secret_not_found(service, key)) from ex
         return value, mtime, owner
 
     def _delete(self, service, key):
         """delete key from service"""
+        if self.force_lower_services:
+            service = service.lower()
+        if self.force_lower_keys:
+            key = key.lower()
         try:
-            (self.secrets_dir / service.lower() / key.lower()).unlink()
+            (self.secrets_dir / service / key).unlink()
         except FileNotFoundError as ex:
-            raise ChamberError("Error: secret not found") from ex
+            raise ChamberError(self._secret_not_found(service, key)) from ex
 
 
 class FileChamber(Chamber):
-    def __init__(self, config, debug, echo):
-        super().__init__(config, debug, echo)
-        self.secrets_file = config["file"]
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self.secrets_file = self.config["file"]
         self.dirty = False
 
     def __enter__(self):
@@ -355,7 +451,10 @@ class FileChamber(Chamber):
     def _secrets(self, service):
         s = self.secrets
         for level in service.split("/"):
-            s = s.get(level, {})
+            try:
+                s = s[level]
+            except KeyError:
+                s = {}
         ret = {k: v for k, v in s.items() if not isinstance(v, dict)}
         return ret
 
@@ -403,4 +502,4 @@ class FileChamber(Chamber):
             del s[key]
             self.dirty = True
         except KeyError as ex:
-            raise ChamberError("Error: secret not found") from ex
+            raise ChamberError(self._secret_not_found(service, key)) from ex
