@@ -77,22 +77,6 @@ def find(shared_datadir, testinit_export):
     yield _find
 
 
-@pytest.fixture(scope="function")
-def reset_vault(shared_datadir, testinit_clear, testinit_import):
-    def _reset_vault():
-        datafile = shared_datadir / "secrets.json"
-        testinit_clear(path="testservice")
-        testinit_clear(path="new_service")
-        testinit_import(path="/", json_string=datafile.read_text())
-
-    return _reset_vault
-
-
-@pytest.fixture(scope="function", autouse=True)
-def init_vault(reset_vault):
-    reset_vault()
-
-
 @pytest.fixture
 def lines():
     def _lines(c):
@@ -134,7 +118,7 @@ def errors():
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_list_services(chamber_class, config, lines, capsys):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         capsys.readouterr()
         ret = chamber.list_services()
         assert ret == 0
@@ -150,7 +134,7 @@ def test_chamber_list_services(chamber_class, config, lines, capsys):
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_list_services_filtered(chamber_class, config, lines, capsys):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.list_services(service_filter="testservice/sub1")
     assert ret == 0
     lines = lines(capsys)
@@ -160,7 +144,7 @@ def test_chamber_list_services_filtered(chamber_class, config, lines, capsys):
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_list_services_and_secrets(chamber_class, config, lines, capsys):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.list_services(include_secrets=True)
     assert ret == 0
     lines = [line for line in lines(capsys) if line.startswith("testservice")]
@@ -182,7 +166,7 @@ def test_chamber_list_services_and_secrets(chamber_class, config, lines, capsys)
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_read(chamber_class, config, lines, capsys):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.read("testservice", "key1")
     assert ret == 0
     lines = lines(capsys)
@@ -194,7 +178,7 @@ def test_chamber_read(chamber_class, config, lines, capsys):
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_readsubkey(chamber_class, config, lines, capsys):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.read("testservice/sub2", "key1")
     assert ret == 0
     lines = lines(capsys)
@@ -207,7 +191,7 @@ def test_chamber_readsubkey(chamber_class, config, lines, capsys):
 @pytest.mark.parametrize("chamber_class, find_type", [(EnvdirChamber, "dir"), (FileChamber, "file"), (VaultChamber, "vault")])
 def test_chamber_write(chamber_class, config, find, find_type, lines, capsys):
     before = find(find_type)
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.write("testservice", "test_write_key", "test_write_value")
     assert ret == 0
     after = find(find_type)
@@ -218,7 +202,7 @@ def test_chamber_write(chamber_class, config, find, find_type, lines, capsys):
 @pytest.mark.parametrize("chamber_class, find_type", [(EnvdirChamber, "dir"), (FileChamber, "file"), (VaultChamber, "vault")])
 def test_chamber_delete_exists(chamber_class, config, find, find_type, lines, capsys):
     before = find(find_type)
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.delete("testservice", "key1")
     assert ret == 0
     after = find(find_type)
@@ -227,18 +211,18 @@ def test_chamber_delete_exists(chamber_class, config, find, find_type, lines, ca
 
 @pytest.mark.parametrize("chamber_class, find_type", [(EnvdirChamber, "dir"), (FileChamber, "file"), (VaultChamber, "vault")])
 def test_chamber_delete_notfound(chamber_class, config, find, find_type, lines, capsys):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         with pytest.raises(ChamberError) as exc_info:
             chamber.delete("testservice", "sir_not_appearing_in_this_film")
     assert exc_info
     assert exc_info.type == ChamberError
-    assert exc_info.value.args[0] == "Error: secret not found"
+    assert exc_info.value.args[0] == "Error: secret not found: 'testservice/sir_not_appearing_in_this_film'"
 
 
 @pytest.mark.parametrize("chamber_class, find_type", [(EnvdirChamber, "dir"), (FileChamber, "file"), (VaultChamber, "vault")])
 def test_chamber_prune_exists(chamber_class, config, find, find_type, lines, capsys):
     before = find(find_type)
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.prune("testservice/sub1")
     assert ret == 0
     after = find(find_type)
@@ -248,7 +232,7 @@ def test_chamber_prune_exists(chamber_class, config, find, find_type, lines, cap
 @pytest.mark.parametrize("chamber_class, find_type", [(EnvdirChamber, "dir"), (FileChamber, "file"), (VaultChamber, "vault")])
 def test_chamber_prune_notfound(chamber_class, config, find, find_type, lines, capsys):
     before = find(find_type)
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=False) as chamber:
         ret = chamber.prune("we_are_not_amused")
     assert ret == 0
     after = find(find_type)
@@ -268,7 +252,7 @@ def testservice_env_lines():
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_env(chamber_class, config, lines, capsys, testservice_env_lines):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.env("testservice")
     assert ret == 0
     lines = lines(capsys)
@@ -282,7 +266,7 @@ def verify_json(shared_datadir):
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_export_json(chamber_class, config, capsys, verify_json, output):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.export(fmt="json", compact_json=True, sort_keys=True, service="testservice", output_file=sys.stdout)
     assert ret == 0
     assert output(capsys) == verify_json
@@ -295,7 +279,7 @@ def verify_yaml(shared_datadir):
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_export_yaml(chamber_class, config, capsys, verify_yaml, output):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.export(fmt="yaml", service="testservice", output_file=sys.stdout)
     assert ret == 0
     output_dict = yaml.load(output(capsys), Loader=Loader)
@@ -310,7 +294,7 @@ def verify_csv(shared_datadir):
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_export_csv(chamber_class, config, capsys, verify_csv, output):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.export(fmt="csv", service="testservice", output_file=sys.stdout)
     assert ret == 0
     output_csv = output(capsys)
@@ -324,7 +308,7 @@ def verify_tsv(shared_datadir):
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_export_tsv(chamber_class, config, capsys, verify_tsv, output):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.export(fmt="tsv", service="testservice", output_file=sys.stdout)
     assert ret == 0
     output_tsv = output(capsys)
@@ -338,7 +322,7 @@ def verify_dotenv(shared_datadir):
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_export_dotenv(chamber_class, config, capsys, verify_dotenv, output):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.export(fmt="dotenv", service="testservice", output_file=sys.stdout)
     assert ret == 0
     output_dotenv = output(capsys)
@@ -352,7 +336,7 @@ def verify_tfvars(shared_datadir):
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_export_tfvars(chamber_class, config, capsys, verify_tfvars, output):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.export(fmt="tfvars", service="testservice", output_file=sys.stdout)
     assert ret == 0
     output_tfvars = output(capsys)
@@ -361,7 +345,7 @@ def test_chamber_export_tfvars(chamber_class, config, capsys, verify_tfvars, out
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_list_keys(chamber_class, config, capsys, lines):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.list("testservice/sub2")
     assert ret == 0
     output_lines = lines(capsys)
@@ -387,7 +371,7 @@ def new_service_lines():
 @pytest.mark.parametrize("chamber_class, find_type", [(EnvdirChamber, "dir"), (FileChamber, "file"), (VaultChamber, "vault")])
 def test_chamber_import(chamber_class, config, find, find_type, json_file, new_service_lines):
     before = find(find_type)
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber._import("new_service", json_file.open("r"))
     assert ret == 0
     after = find(find_type)
@@ -407,7 +391,7 @@ def valid_found_lines():
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
 def test_chamber_find(chamber_class, config, lines, capsys, valid_found_lines):
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber.find(by_value=False, key="key1")
     assert ret == 0
     out_lines = lines(capsys)
@@ -425,12 +409,13 @@ def test_chamber_exec(chamber_class, config, lines, capfd, testservice_json):
 
     after_cmd = ["env"]
     capfd.readouterr()
-    with chamber_class(config, True, _echo) as chamber:
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         ret = chamber._exec(
             pristine=True,
             strict_value=None,
             services=["testservice"],
             cmd=after_cmd,
+            buffer_output=False
         )
     assert ret == 0
     after = lines(capfd)
@@ -439,19 +424,19 @@ def test_chamber_exec(chamber_class, config, lines, capfd, testservice_json):
     pprint(diff)
 
     verify_testservice = json.loads(testservice_json.read_text())
-    verify_testservice = {k: v for k, v in verify_testservice.items()}
+    verify_testservice = {k.upper(): v for k, v in verify_testservice.items()}
 
     diff_dict = {}
     for line in diff:
         key, _, value = line.partition("=")
-        diff_dict[key.lower()] = value
+        diff_dict[key] = value
 
     assert diff_dict == verify_testservice
 
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
-def test_exec_bad_command(chamber_class, config):
-    with chamber_class(config, True, _echo) as chamber:
+def test_chamber_exec_bad_command(chamber_class, config):
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         _cmd = ["nonexistent_command"]
         with pytest.raises(Exception) as exc_info:
             chamber._exec(pristine=True, strict_value=None, services=["testservice"], cmd=_cmd)
@@ -459,10 +444,19 @@ def test_exec_bad_command(chamber_class, config):
 
 
 @pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
-def test_exec_error_command(chamber_class, config, capfd):
-    with chamber_class(config, True, _echo) as chamber:
+def test_chamber_exec_nonexistent_service(chamber_class, config):
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
+        _cmd = ["bash", "-c", "env"]
+        with pytest.raises(Exception) as exc_info:
+            chamber._exec(pristine=True, strict_value=None, services=["nonexistent_service"], cmd=_cmd)
+        print(f"Exception: {exc_info}")
+
+
+@pytest.mark.parametrize("chamber_class", [EnvdirChamber, FileChamber, VaultChamber])
+def test_chamber_exec_error_command(chamber_class, config, capfd):
+    with chamber_class(config=config, debug=True, echo=_echo, require_exists=True) as chamber:
         _cmd = ["bash", "-c", "ls --nonexistent_option"]
-        ret = chamber._exec(pristine=True, strict_value=None, services=["testservice"], cmd=_cmd)
+        ret = chamber._exec(buffer_output=False, pristine=True, strict_value=None, services=["testservice"], cmd=_cmd)
     assert ret != 0, "expected non-zero return"
     out = capfd.readouterr()
     assert out.err
